@@ -38,6 +38,8 @@
 #include <xmltooling/util/Threads.h>
 #include <xmltooling/util/XMLHelper.h>
 #include <xmltooling/validation/ValidatorSuite.h>
+#include <xmltooling/security/SecurityHelper.h>
+
 
 #if defined(XMLTOOLING_LOG4SHIB)
 # include <log4shib/NDC.hh>
@@ -50,9 +52,14 @@ using namespace xmltooling::logging;
 using namespace xmltooling;
 using namespace std;
 
+#include <limits>
+
 # ifndef min
 #  define min(a,b)            (((a) < (b)) ? (a) : (b))
 # endif
+# ifdef max
+# undef max
+# endif
 
 static const XMLCh id[] =                   UNICODE_LITERAL_2(i,d);
 static const XMLCh cleanupInterval[] =      UNICODE_LITERAL_15(c,l,e,a,n,u,p,I,n,t,e,r,v,a,l);
@@ -72,7 +79,7 @@ namespace opensaml {
 };
 
 DynamicMetadataProvider::DynamicMetadataProvider(const DOMElement* e)
-    : AbstractMetadataProvider(e),
+    : AbstractMetadataProvider(e), MetadataProvider(e),
       m_validate(XMLHelper::getAttrBool(e, false, validate)),
         m_id(XMLHelper::getAttrString(e, "Dynamic", id)),
         m_lock(RWLock::create()),
@@ -85,7 +92,7 @@ DynamicMetadataProvider::DynamicMetadataProvider(const DOMElement* e)
         m_cleanup_wait(nullptr), m_cleanup_thread(nullptr)
 {
     if (m_minCacheDuration > m_maxCacheDuration) {
-        Category::getInstance(SAML_LOGCAT ".MetadataProvider.Dynamic").error(
+        Category::getInstance(SAML_LOGCAT ".Metadata.Dynamic").error(
             "minCacheDuration setting exceeds maxCacheDuration setting, lowering to match it"
             );
         m_minCacheDuration = m_maxCacheDuration;
@@ -280,6 +287,14 @@ pair<const EntityDescriptor*,const RoleDescriptor*> DynamicMetadataProvider::get
             log.error("metadata instance did not match expected entityID");
             return entity;
         }
+        else if (criteria.artifact) {
+            auto_ptr_char temp2(entity2->getEntityID());
+            const string hashed(SecurityHelper::doHash("SHA1", temp2.get(), strlen(temp2.get()), true));
+            if (hashed != name) {
+                log.error("metadata instance did not match expected entityID");
+                return entity;
+            }
+        }
         else {
             auto_ptr_XMLCh temp2(name.c_str());
             if (!XMLString::equals(temp2.get(), entity2->getEntityID())) {
@@ -301,7 +316,10 @@ pair<const EntityDescriptor*,const RoleDescriptor*> DynamicMetadataProvider::get
         doFilters(*entity2);
 
         time_t now = time(nullptr);
-        if (entity2->getValidUntil() && entity2->getValidUntilEpoch() < now + 60)
+        time_t cmp = now;
+        if (cmp < (std::numeric_limits<int>::max() - 60))
+            cmp += 60;
+        if (entity2->getValidUntil() && entity2->getValidUntilEpoch() < cmp)
             throw MetadataException("Metadata was already invalid at the time of retrieval.");
 
         log.info("caching resolved metadata for (%s)", name.c_str());
